// sudo apt install dotnet-sdk-8.0
// dotnet build loops.csproj && ./bin/Debug/netcoreapp*/loops
// dotnet publish loops.csproj && ./bin/Release/netcoreapp*/loops

using System;
using System.Collections.Generic;
using System.Linq;

namespace code
{
    class Logic
    {
        static long WithLinq(List<long> values)
        {
            long total = values
                .Where(v => v % 2 == 0)
                .Select(v => v / 2)
                .Aggregate(0L, (acc, v) => acc + v);  // or .Sum()
            return total;
        }
        static long WithLoop(List<long> values)
        {
            long total = 0;
            foreach (long v in values) {
                if (v % 2 == 0) {
                    total += v / 2;
                }
            }
            return total;
        }
        static long WithIndex(List<long> values)
        {
            long total = 0;
            for (int i=0; i < values.Count; i++) {
                long v = values[i];
                if (v % 2 == 0) {
                    total += v / 2;
                }
            }
            return total;
        }
        static long WithLinqParallel(List<long> values)
        {
            long total = values.AsParallel()
                .Where(v => v % 2 == 0)
                .Select(v => v / 2)
                .Aggregate(0L, (acc, v) => acc + v);  // or .Sum()
            return total;
        }
        
        static void Main(string[] args)
        {
            Random rand = new Random();
            List<long> values = Enumerable.Range(0, 1000000000).Select(_ => (long)rand.Next(1, 10)).ToList();
            
            var start = DateTime.Now;
            var result = Logic.WithLinq(values);
            var end = DateTime.Now;
            Console.Write($"Result: {result}\n");
            Console.Write($"Elapsed: {end.Subtract(start).ToString()}\n");
            
        }
    }
}